@@ -37,11 +37,14 @@ from .const import (
     REST_PATH_SYSTEMINFO,
     REST_RECORDING_DETAILS,
     SKY_PLAY_URN,
+    SKY_STATE_NOMEDIA,
     SKY_STATE_OFF,
     SKY_STATE_ON,
     SKY_STATE_PAUSED,
     SKY_STATE_PLAYING,
     SKY_STATE_STANDBY,
+    SKY_STATE_STOPPED,
+    SKY_STATE_TRANSITIONING,
     SKYCONTROL,
     SOAP_ACTION,
     SOAP_CONTROL_BASE_URL,
@@ -82,7 +85,9 @@ class SkyQRemote:
         self._channel = None
         self._lastEpg = None
         self._programme = None
+        self._recordedProgramme = None
         self._lastProgrammeEpg = None
+        self._lastPvrId = None
         self._currentApp = APP_EPG
         self._channels = []
         self._error = False
@@ -91,12 +96,8 @@ class SkyQRemote:
 
     def powerStatus(self) -> str:
         """Get the power status of the Sky Q box."""
-        if not self.deviceSetup:
-            self._setupDevice()
-
         if not self._remoteCountry:
-            SkyQCountry = self._importCountry(self._epgCountryCode)
-            self._remoteCountry = SkyQCountry()
+            self._setupRemote()
 
         if self._soapControlURL is None:
             return SKY_STATE_OFF
@@ -111,18 +112,22 @@ class SkyQRemote:
 
     def getCurrentState(self):
         """Get current state of the SkyQ box."""
-        if self.powerStatus() == SKY_STATE_OFF:
+        if not self._remoteCountry:
+            self._setupRemote()
+
+        if self._soapControlURL is None:
             return SKY_STATE_OFF
-        if self.powerStatus() == SKY_STATE_STANDBY:
-            return SKY_STATE_STANDBY
+
         response = self._callSkySOAPService(UPNP_GET_TRANSPORT_INFO)
         if response is not None:
             state = response[CURRENT_TRANSPORT_STATE]
-            if state == SKY_STATE_PLAYING:
+            if state == SKY_STATE_NOMEDIA or state == SKY_STATE_STOPPED:
+                return SKY_STATE_STANDBY
+            if state == SKY_STATE_PLAYING or state == SKY_STATE_TRANSITIONING:
                 return SKY_STATE_PLAYING
             if state == SKY_STATE_PAUSED:
                 return SKY_STATE_PAUSED
-        return SKY_STATE_STANDBY
+        return SKY_STATE_OFF
 
     def getActiveApplication(self):
         """Get the active application on Sky Q box."""
@@ -219,7 +224,7 @@ class SkyQRemote:
             if not self._error:
                 self._error = True
                 _LOGGER.info(
-                    f"I0020 - Programme data not found for SID {sid} : {epgDate}"
+                    f"I0020 - Programme data not found for host: {self._host}/{self._overrideCountry} sid: {sid} : {epgDate}"
                 )
                 return EPG_ERROR_NO_DATA
         else:
@@ -274,6 +279,10 @@ class SkyQRemote:
         imageUrl = None
         title = None
 
+        if self._lastPvrId == pvrId:
+            return self._recordedProgramme
+        self._lastPvrId = pvrId
+
         resp = self._http_json(REST_RECORDING_DETAILS.format(pvrId))
         if "details" not in resp:
             _LOGGER.info(f"I0030 - Recording data not found for {pvrId}")
@@ -301,11 +310,11 @@ class SkyQRemote:
         else:
             endtime = starttime
 
-        programme = RecordedProgramme(
+        self._recordedProgramme = RecordedProgramme(
             programmeuuid, starttime, endtime, title, season, episode, imageUrl, channel
         )
 
-        return programme
+        return self._recordedProgramme
 
     def getDeviceInformation(self):
         """Get the device information from the SkyQ box."""
@@ -366,7 +375,7 @@ class SkyQRemote:
             channel = Channel(channelno, channelname, sf=sf)
             channelitems.add(channel)
 
-        channelnosorted = sorted(channelitems, key=attrgetter("channelno"))
+        channelnosorted = sorted(channelitems, key=attrgetter("channelnoint"))
         self._channellist = ChannelList(
             sorted(channelnosorted, key=attrgetter("channeltype"), reverse=True)
         )
@@ -564,12 +573,21 @@ class SkyQRemote:
     def _getNodeFromChannels(self, sid):
         return next((s for s in self._channels if s["sid"] == str(sid)), None)
 
-    def _setupDevice(self):
-        """Set the remote up."""
+    def _setupRemote(self):
         deviceInfo = self.getDeviceInformation()
         if not deviceInfo:
             return
 
+        if not self.deviceSetup:
+            self._setupDevice()
+
+        """Set the remote up."""
+        if not self._remoteCountry and self.deviceSetup:
+            SkyQCountry = self._importCountry(self._epgCountryCode)
+            self._remoteCountry = SkyQCountry()
+
+    def _setupDevice(self):
+
         url_index = 0
         self._soapControlURL = None
         while self._soapControlURL is None and url_index < 50:
